@@ -25,8 +25,9 @@ use crate::{
     source::{Combined, Source},
 };
 
-use self::stats::Stats;
+use self::{budget::Budget, stats::Stats};
 
+mod budget;
 mod stats;
 
 static DUMPER: Lazy<Dumper> = Lazy::new(|| Dumper::new(INTERNAL_CLASS));
@@ -42,6 +43,7 @@ pub struct Context<C = (), K = Singleton, S = ()> {
     source: S,
     stage: Stage,
     stats: Stats,
+    budget: Budget,
 }
 
 #[derive(Clone, Copy, PartialEq)]
@@ -98,6 +100,7 @@ impl<C, K, S> Context<C, K, S> {
             source: Combined::new(self.source, source),
             stage: Stage::PreRecv,
             stats: Stats::default(),
+            budget: self.budget,
         }
     }
 
@@ -170,7 +173,7 @@ impl<C, K, S> Context<C, K, S> {
         self.stats.sent_messages_total::<M>();
 
         trace!("> {:?}", message);
-        if let Some(permit) = DUMPER.acquire() {
+        if let Some(permit) = DUMPER.acquire_m::<M>() {
             permit.record(Dump::message(message.clone(), &kind, Direction::Out));
         }
 
@@ -260,7 +263,7 @@ impl<C, K, S> Context<C, K, S> {
         self.stats.sent_messages_total::<M>();
 
         trace!("> {:?}", message);
-        if let Some(permit) = DUMPER.acquire() {
+        if let Some(permit) = DUMPER.acquire_m::<M>() {
             permit.record(Dump::message(message.clone(), &kind, Direction::Out));
         }
 
@@ -343,7 +346,7 @@ impl<C, K, S> Context<C, K, S> {
         self.stats.sent_messages_total::<M>();
 
         trace!(to = %recipient, "> {:?}", message);
-        if let Some(permit) = DUMPER.acquire() {
+        if let Some(permit) = DUMPER.acquire_m::<M>() {
             permit.record(Dump::message(message.clone(), &kind, Direction::Out));
         }
 
@@ -382,7 +385,7 @@ impl<C, K, S> Context<C, K, S> {
         let kind = MessageKind::Regular { sender: self.addr };
 
         trace!(to = %recipient, "> {:?}", message);
-        if let Some(permit) = DUMPER.acquire() {
+        if let Some(permit) = DUMPER.acquire_m::<M>() {
             permit.record(Dump::message(message.clone(), &kind, Direction::Out));
         }
 
@@ -421,7 +424,7 @@ impl<C, K, S> Context<C, K, S> {
         };
 
         trace!(to = %sender, "> {:?}", message);
-        if let Some(permit) = DUMPER.acquire() {
+        if let Some(permit) = DUMPER.acquire_m::<R>() {
             permit.record(Dump::message(message.clone(), &kind, Direction::Out));
         }
 
@@ -451,6 +454,9 @@ impl<C, K, S> Context<C, K, S> {
         C: 'static,
         S: Source,
     {
+        // TODO: reset if the mailbox is empty.
+        self.budget.acquire().await;
+
         loop {
             self.stats.message_handling_time_seconds();
 
@@ -573,18 +579,20 @@ impl<C, K, S> Context<C, K, S> {
         let message = envelope.message();
         trace!("< {:?}", message);
 
-        if let Some(permit) = DUMPER.acquire() {
-            // TODO: reuse `Dump::message`, it requires `AnyMessage: Message`.
-            let dump = Dump::builder()
-                .direction(Direction::In)
-                .message_name(message.name())
-                .message_protocol(message.protocol())
-                .message_kind(dumping::MessageKind::from_message_kind(
-                    envelope.message_kind(),
-                ))
-                .do_finish(message.erase());
+        if message.dumping_allowed() {
+            if let Some(permit) = DUMPER.acquire() {
+                // TODO: reuse `Dump::message`, it requires `AnyMessage: Message`.
+                let dump = Dump::builder()
+                    .direction(Direction::In)
+                    .message_name(message.name())
+                    .message_protocol(message.protocol())
+                    .message_kind(dumping::MessageKind::from_message_kind(
+                        envelope.message_kind(),
+                    ))
+                    .do_finish(message.erase());
 
-            permit.record(dump);
+                permit.record(dump);
+            }
         }
 
         // We should change the status after dumping the original message
@@ -641,6 +649,7 @@ impl<C, K, S> Context<C, K, S> {
             source: (),
             stage: self.stage,
             stats: Stats::default(),
+            budget: self.budget.clone(),
         }
     }
 
@@ -659,6 +668,7 @@ impl<C, K, S> Context<C, K, S> {
             source: self.source,
             stage: self.stage,
             stats: Stats::default(),
+            budget: self.budget,
         }
     }
 
@@ -683,6 +693,7 @@ impl<C, K, S> Context<C, K, S> {
             source: self.source,
             stage: self.stage,
             stats: Stats::default(),
+            budget: self.budget,
         }
     }
 }
@@ -726,6 +737,7 @@ impl Context {
             source: (),
             stage: Stage::PreRecv,
             stats: Stats::default(),
+            budget: Budget::default(),
         }
     }
 }
@@ -742,6 +754,7 @@ impl<C, K: Clone> Clone for Context<C, K> {
             source: (),
             stage: self.stage,
             stats: Stats::default(),
+            budget: self.budget.clone(),
         }
     }
 }
@@ -836,7 +849,7 @@ impl<'c, C: 'static, K, S, R: Request> RequestBuilder<'c, C, S, K, R, Any> {
 
             // TODO: increase a counter.
             trace!("< {:?}", envelope.message());
-            if let Some(permit) = DUMPER.acquire() {
+            if let Some(permit) = DUMPER.acquire_m::<R>() {
                 permit.record(Dump::message(
                     envelope.message().clone(),
                     envelope.message_kind(),
@@ -890,7 +903,7 @@ impl<'c, C: 'static, K, S, R: Request> RequestBuilder<'c, C, K, S, R, All> {
                 trace!("< {:?}", envelope.message());
 
                 // TODO: `acquire_many` or even unconditionally?
-                if let Some(permit) = DUMPER.acquire() {
+                if let Some(permit) = DUMPER.acquire_m::<R>() {
                     permit.record(Dump::message(
                         envelope.message().clone(),
                         envelope.message_kind(),
