@@ -1,5 +1,7 @@
 use std::sync::Arc;
 
+use crate::Message;
+
 use super::{
     dump::*,
     recorder::{self, Recorder},
@@ -24,6 +26,14 @@ impl Dumper {
         let r = self.recorder.as_deref().filter(|r| r.enabled())?;
         Some(DumpingPermit { recorder: &*r })
     }
+
+    pub(crate) fn acquire_m<M: Message>(&self) -> Option<DumpingPermit<'_>> {
+        if !M::VTABLE.dumping_allowed {
+            return None;
+        }
+
+        self.acquire()
+    }
 }
 
 #[must_use]
