@@ -6,7 +6,8 @@ use std::{
 
 use serde::Serialize;
 
-#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Serialize)]
+// TODO: make it just type alias (or not?)
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Serialize)]
 pub struct SequenceNo(NonZeroU64);
 
 impl TryFrom<u64> for SequenceNo {
