@@ -8,20 +8,9 @@ use smallbox::{smallbox, SmallBox};
 
 use crate::dumping;
 
-pub type LocalTypeId = u32;
-
 pub trait Message: fmt::Debug + Clone + Any + Send + Serialize + for<'de> Deserialize<'de> {
     #[doc(hidden)]
-    const _LTID: LocalTypeId;
-
-    /// A protocol's name.
-    /// Usually, it's a crate name where the message is defined.
-    const PROTOCOL: &'static str;
-    /// Just a message's name.
-    const NAME: &'static str;
-
-    #[doc(hidden)]
-    const LABELS: &'static [Label];
+    const VTABLE: &'static MessageVTable;
 
     // Called while upcasting/downcasting to avoid
     // [rust#47384](https://github.com/rust-lang/rust/issues/47384).
@@ -37,7 +26,7 @@ pub trait Request: Message {
 }
 
 pub struct AnyMessage {
-    ltid: LocalTypeId,
+    vtable: &'static MessageVTable,
     data: SmallBox<dyn Any + Send, [u8; 184]>,
 }
 
@@ -47,25 +36,31 @@ impl AnyMessage {
         message._touch();
 
         AnyMessage {
-            ltid: M::_LTID,
+            vtable: M::VTABLE,
             data: smallbox!(message),
         }
     }
 
     #[inline]
     pub fn name(&self) -> &'static str {
-        with_vtable(self.ltid, |vtable| vtable.name)
+        self.vtable.name
     }
 
     #[inline]
     pub fn protocol(&self) -> &'static str {
-        with_vtable(self.ltid, |vtable| vtable.protocol)
+        self.vtable.protocol
     }
 
     #[inline]
     #[doc(hidden)]
     pub fn labels(&self) -> &'static [Label] {
-        with_vtable(self.ltid, |vtable| vtable.labels)
+        self.vtable.labels
+    }
+
+    #[inline]
+    #[doc(hidden)]
+    pub fn dumping_allowed(&self) -> bool {
+        self.vtable.dumping_allowed
     }
 
     #[inline]
@@ -83,7 +78,7 @@ impl AnyMessage {
 
     #[inline]
     pub fn downcast<M: Message>(self) -> Result<M, AnyMessage> {
-        if M::_LTID != self.ltid {
+        if !self.is::<M>() {
             return Err(self);
         }
 
@@ -100,20 +95,20 @@ impl AnyMessage {
     #[inline]
     #[doc(hidden)]
     pub fn erase(&self) -> dumping::ErasedMessage {
-        with_vtable(self.ltid, |vtable| (vtable.erase)(self))
+        (self.vtable.erase)(self)
     }
 }
 
 impl Clone for AnyMessage {
     #[inline]
     fn clone(&self) -> Self {
-        with_vtable(self.ltid, |vtable| (vtable.clone)(self))
+        (self.vtable.clone)(self)
     }
 }
 
 impl fmt::Debug for AnyMessage {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        with_vtable(self.ltid, |vtable| (vtable.debug)(self, f))
+        (self.vtable.debug)(self, f)
     }
 }
 
@@ -122,32 +117,29 @@ impl fmt::Debug for AnyMessage {
 // Reexported in `elfo::_priv`.
 #[derive(Clone)]
 pub struct MessageVTable {
-    pub ltid: LocalTypeId,
+    /// Just a message's name.
     pub name: &'static str,
+    /// A protocol's name.
+    /// Usually, it's a crate name where the message is defined.
     pub protocol: &'static str,
     pub labels: &'static [Label],
+    pub dumping_allowed: bool, // TODO: introduce `DumpingMode`.
     pub clone: fn(&AnyMessage) -> AnyMessage,
     pub debug: fn(&AnyMessage, &mut fmt::Formatter<'_>) -> fmt::Result,
     pub erase: fn(&AnyMessage) -> dumping::ErasedMessage,
 }
 
 #[distributed_slice]
-pub static MESSAGE_LIST: [MessageVTable] = [..];
+pub static MESSAGE_LIST: [&'static MessageVTable] = [..];
 
 thread_local! {
-    // TODO: access it speculatively during initialization.
-    // TODO: use simd + `SmallVec<[Vec<MessageVTable>; N]>` and sequential LTIDs.
-    static MESSAGE_BY_LTID: FxHashMap<LocalTypeId, MessageVTable> = {
+    static MESSAGE_BY_NAME: FxHashMap<(&'static str, &'static str), &'static MessageVTable> = {
         MESSAGE_LIST.iter()
-            .map(|vtable| (vtable.ltid, vtable.clone()))
+            .map(|vtable| ((vtable.protocol, vtable.name), *vtable))
             .collect()
     };
 }
 
-fn with_vtable<R>(ltid: LocalTypeId, f: impl FnOnce(&MessageVTable) -> R) -> R {
-    MESSAGE_BY_LTID.with(|map| f(map.get(&ltid).expect("invalid LTID")))
-}
-
 pub(crate) fn init() {
-    MESSAGE_BY_LTID.with(|_| ());
+    MESSAGE_BY_NAME.with(|_| ());
 }
