@@ -20,6 +20,8 @@ pub struct Scope {
     meta: Arc<ActorMeta>,
     trace_id: Cell<TraceId>,
     shared: Arc<ScopeShared>,
+    allocated_bytes: Cell<usize>,
+    deallocated_bytes: Cell<usize>,
 }
 
 assert_impl_all!(Scope: Send);
@@ -48,6 +50,8 @@ impl Scope {
             meta,
             trace_id: Cell::new(trace_id),
             shared,
+            allocated_bytes: Cell::new(0),
+            deallocated_bytes: Cell::new(0),
         }
     }
 
@@ -107,6 +111,27 @@ impl Scope {
         &self.shared.dumping
     }
 
+    #[doc(hidden)]
+    #[stability::unstable]
+    pub fn increment_allocated_bytes(&self, by: usize) {
+        self.allocated_bytes.set(self.allocated_bytes.get() + by);
+    }
+
+    #[doc(hidden)]
+    #[stability::unstable]
+    pub fn increment_deallocated_bytes(&self, by: usize) {
+        self.deallocated_bytes
+            .set(self.deallocated_bytes.get() + by);
+    }
+
+    pub(crate) fn take_allocated_bytes(&self) -> usize {
+        self.allocated_bytes.take()
+    }
+
+    pub(crate) fn take_deallocated_bytes(&self) -> usize {
+        self.deallocated_bytes.take()
+    }
+
     /// Wraps the provided future with the current scope.
     pub async fn within<F: Future>(self, f: F) -> F::Output {
         SCOPE.scope(self, f).await
